/*
 * Project: irond
 * Package: src.de.fhhannover.inform.iron.mapserver.datamodel
 * File:    Searcher.java
 *
 * Copyright (C) 2010-2011 Fachhochschule Hannover
 * Ricklinger Stadtweg 118, 30459 Hannover, Germany 
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.fhhannover.inform.iron.mapserver.datamodel;

import java.util.List;

import org.apache.log4j.Logger;

import de.fhhannover.inform.iron.mapserver.datamodel.graph.Identifier;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.Link;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.MetadataContainer;
import de.fhhannover.inform.iron.mapserver.datamodel.graph.meta.Metadata;
import de.fhhannover.inform.iron.mapserver.exceptions.SearchException;
import de.fhhannover.inform.iron.mapserver.exceptions.SearchResultsTooBigException;
import de.fhhannover.inform.iron.mapserver.logging.Logging;
import de.fhhannover.inform.iron.mapserver.messages.SearchRequest;
import de.fhhannover.inform.iron.mapserver.provider.DataModelServerConfigurationProvider;
import de.fhhannover.inform.iron.mapserver.utils.LinkedMap;
import de.fhhannover.inform.iron.mapserver.utils.NullCheck;

/**
 * This class does the real searching using a {@link SearchResultBuilder}.
 * 
 * TODO: terminal-identifier-types is only string comparison, could be improved.
 * 
 * @author aw
 * @version 0.1
 */

/*
 * created: 09.12.09
 * changes:
 *	09.12.09 aw - Created first version of this class, visibility package
 *		should be enough, prototype search algorithm implemented.
 *	17.12.09 aw - Added terminal-identifier-type return. Should be tested.
 *	17.12.09 aw - terminalIdentifierTypes wasn't set.
 *  04.02.10 aw - added log4j Logger, transformed log calls
 *  05.02.10 aw - use two list which are added into the SearchResult at a
 *                later time
 *  11.02.10 vp - the visibility of the classes and methods is changed
 *  12.02.10 aw - general code cleanup and commenting,
 *                more verbose debugging for the search algorithm
 *                fixed missing depth++ in traverse()
 *  30.04.10 aw - remove singletons!
 *  01.05.10 aw - prototype for Director/Builder pattern, Use a SearchResultBuilder
 *  			  to construct the SearchResult. Don't use two lists and so on...
 *
 */
class Searcher {
	
	private static Logger logger;
	static {
		logger = Logging.getTheLogger();
	}

	/*
	 * The following members can be extracted from the request 
	 */
	private final int maxDepth;
	private final int maxResultSize;
	private final Identifier startIdent;
	private final String termIdentTypes;
	private final Filter matchLinksFilter;
	private final Filter resultFilter;
	private final IdentifierRep identifierRep;
	private boolean ignoreSize = false;
	
	/**
	 * containers have been visited before
	 */
	private final LinkedMap<MetadataContainer> visitedContainers;
	
	/**
	 * the size to be added when determining whether the result has grown to big;
	 */
	private int addBytes;
	
	private final DataModelServerConfigurationProvider dataModelConf;
	
	
	/**
	 * The {@link SearchResultBuilder} to be used.
	 */
	private SearchResultBuilder searchBuilder;
	
	/**
	 * Instantiate a Searcher based upon a SearchRequest
	 * 
	 * @param iRep	reference to the {@link IdentifierRep} instance
	 * @param request the original {@link SearchRequest}
	 * @param builder	{@link SearchResultBuilder} to build the result
	 * @param additionalBytes bytes to be added to the current size, useful
	 *        for the enclosing searchResult, updateResult, deleteResult elements,
	 *        maybe
	 */
	Searcher(IdentifierRep iRep, SearchRequest request, SearchResultBuilder builder,
			int additionalBytes) {
		
		dataModelConf = DataModelService.getServerConfiguration();
		identifierRep				= iRep;
		maxDepth					= request.getMaxDepth();
		searchBuilder				= builder;
		matchLinksFilter			= new Filter(request.getMatchLinksFilter());
		resultFilter				= new Filter(request.getResultFilter());
		// get the identifier out of the repository
		startIdent = identifierRep.getOrCreateIdentifier(request.getStartIdentifier());
		
		
		if (request.getTerminalIdentifierTypes() != null) {
			termIdentTypes = new String(request.getTerminalIdentifierTypes());
		} else {
			termIdentTypes = null;
		}
		
		if (!request.maxSizeGiven()) {
			maxResultSize = dataModelConf.getDefaultMaxSearchResultSize();
		} else {
			maxResultSize = request.getMaxResultSize();
		}
		// initial size
		addBytes = additionalBytes;
		
		visitedContainers = new LinkedMap<MetadataContainer>();

	}

	Searcher(IdentifierRep iRep, SearchRequest request, SearchResultBuilder builder) {
		this(iRep, request, builder, 0);
	}

	
	/**
	 * Method where the recursion of the search algorithm is started.
	 * Furthermore the result is filtered here.
	 * 
	 * @throws SearchException
	 */
	void runSearch() throws SearchResultsTooBigException {
		logger.debug("running search with parameters:\n"
				+ "\tstart=" + startIdent + "\n"
				+ "\tmaxDepth=" + maxDepth + "\n"
				+ "\tmaxSize=" + maxResultSize + " byte\n"
				+ "\tmatch-links-filter=" + matchLinksFilter + "\n"
				+ "\tresult-filter=" + resultFilter + "\n"
				+ "\tterminal-identifier-types=" + termIdentTypes + "\n"
				+ "\tignoreSize=" + ignoreSize);
				
		logger.trace("Beginning search at " + startIdent);
		traverse(startIdent, 0, null);
		searchBuilder.filter(resultFilter);

		// final check, we might have missed something
		if (resultIsTooBig()) {
			logger.trace("result ended up being too big");
				throw new SearchResultsTooBigException("too big! ("
						+ usedBytesString() + ")");
		}
		logger.trace("Finished searching (" + usedBytesString() + ")");
	}
	

	void runSearchIgnoreSize() {
		ignoreSize = true;
		try {
			runSearch();
		} catch (SearchResultsTooBigException e) {
			logger.error("UNEXPECTED: SearchResultsTooBig when ignoring size");
			throw new RuntimeException("Programming Error, SearchResultsTooBig" +
					" should never occur in this case");
		}
	}
	
	/**
	 * This part represents the "real" algorithm.
	 * 
	 * The search algorithm is best looked up in the
	 * Specification. But a short overview is given here:
	 * 
	 * - Add the current {@link Identifier} with its {@link Metadata} objects
	 *   
	 * - Check the depth
	 * 
	 * - Check if the current {@link Identifier} object has a type which
	 *   is in the list of the terminal identifier types
	 *   
	 * - Visit each {@link Link} of the current {@link Identifier} match
	 *   all {@link Metadata} objects against the match links filter.
	 *   
	 * - If any {@link Metadata} object matched the match links filter add this
	 *   {@link Link} object with every matched metadata.
	 *   
	 * - Visit the "other end" of the {@link Link} and begin the
	 *   search algorithm with this {@link Identifier} and a depth
	 *   incremented by one.
	 * 
	 * @param cur
	 * @param depth
	 * @param prev
	 * @throws SearchResultsTooBigException 
	 */
	private void traverse(Identifier cur, int depth, Identifier prev)
											throws SearchResultsTooBigException {
		logger.trace("current depth=" + depth);
		NullCheck.check(cur, "search went wrong");
		
		List<Link> links = cur.getLinks();
		List<Metadata> mlist = null;
		Identifier next = null;
		
		visitedContainers.put(cur.hashCode(), cur);
		
		appendToResult(cur);

		
		// If the search result grew too big try to make it smaller
		// remove all unwanted metadata elements from it. If it's
		// still too big, abort.
		logger.trace("size= " + usedBytesString());
		if (resultIsTooBig()) {
			logger.trace("result too big, trying to filter it");
			searchBuilder.filter(resultFilter);
			logger.trace("size now= " + usedBytesString());
			
			if (resultIsTooBig()) {
				logger.trace("still too big!");
				throw new SearchResultsTooBigException("Too big! ("
						+ usedBytesString() + ")");
			}
		}
		
		if (depth >= maxDepth) {
			logger.trace("maximum depth reached");
			return;
		}
		
		if (isTerminalIdentifier(cur)) {
			logger.trace(cur + " is a terminal identifier");
			return;
		}
		
		logger.trace("current identfier has " + links.size() + " links");
		for (Link l : links) {
			
			mlist = l.getMetadataInGraph(matchLinksFilter);
			logger.trace("link has n=" + l.getMetadata().size() +
					" with n=" + mlist.size() + " matching");
			
			if (mlist != null && mlist.size() > 0) {
				next = l.getOtherIdent(cur);
				logger.trace("looking at link from " + cur + " to " + next);
				
				if (next == null || next == prev) {
					logger.trace("this is the one we came from, continue...");
					continue;
				}
				
				// If we traveled this link some time before, we don't
				// need to add the metadata to the result neither do we
				// have to travel it again.
				if (visitedContainers.get(l.hashCode()) != null)
					continue;
			
				// if we didn't travel along this link, we have to add the
				// metadata attached to it, and add it to the visited containers
				appendToResult(l, mlist);
				visitedContainers.put(l.hashCode(), l);
				
				// If the next identifier was visited before, we don't to visit
				// it again, don't travel along this edge.
				if (visitedContainers.get(next.hashCode()) != null)
					continue;
				
				logger.trace("traversing to " + next);
				traverse(next, depth + 1, cur);
			}
		}
	}
	 

	
	/**
	 * Take this {@link Identifier} and forward it together with it's
	 * Metadata to a {@link SearchResultBuilder}. If its list of Metadata
	 * is empty or contains only Metadata which was not in the graph
	 * then add only the {@link Identifier} to the result.
	 * 
	 * @param cur
	 */
	private void appendToResult(Identifier ident) {
		boolean addedMetadata = false;
		for (Metadata m : ident.getMetadataInGraph()) {
			addedMetadata = true;
			searchBuilder.addMetadata(ident, m);
		}
		
		if (!addedMetadata) {
			searchBuilder.addMetadataContainer(ident);
		}
	}
	
	/**
	 * Same as appendToResult() but for links. In the special case of links
	 * only part of their metadata list is added to the searchresult. This
	 * part must me given in mlist.
	 * 
	 * @param link
	 * @param mlist
	 */
	private void appendToResult(Link link, List<Metadata> mlist) {
		boolean addedMetadata = false;
		for (Metadata m : mlist) {
			addedMetadata = true;
			searchBuilder.addMetadata(link, m);
		}
		if (!addedMetadata) {
			searchBuilder.addMetadataContainer(link);
		}
	}

	/**
	 * Helper method to check whether the given identifier is
	 * a terminal-identifier-type.
	 * 
	 * Realized with contains() from the String class
	 * 
	 * @param cur
	 * @return
	 */
	private boolean isTerminalIdentifier(Identifier cur) {
		return termIdentTypes != null
			&& termIdentTypes.contains(cur.getType().getTypeString());
	}

	/**
	 * Simple helper to check if the result grew too big.
	 * 
	 * @return true if the result has grown too big already
	 */
	private boolean resultIsTooBig() {
		return (!ignoreSize && curByteCount() > maxResultSize);
	}

	/**
	 * Simple helper to get the current size of the result.
	 * 
	 * @return
	 */
	private int curByteCount() {
			return searchBuilder.getByteCount() + addBytes;
	}
	
	private String usedBytesString() {
		return String.format("%d / %s bytes", curByteCount(),
				ignoreSize ? "unlimited" : maxResultSize);
		
	}
}
 
